@@ -311,11 +311,12 @@ static char *table_key_completions (const char *text, int state)
 
 static char *generator (const char *text, int state)
 {
-    static int which;
+    static int which, completed;
     char *match = NULL;
 
     if (state == 0) {
 	which = 0;
+	completed = 0;
     }
 
     /* Try to complete a keyword. */
@@ -323,10 +324,12 @@ static char *generator (const char *text, int state)
     if (which == 0) {
 	match = keyword_completions (text, state);
 
-	if (!match) {
+	if (!match /* && !completed */) {
 	    which = 1;
 	    state = 0;
-	}
+	} else {
+            completed = 1;
+        }
     } 
 
     /* Try to complete a table access. */
@@ -334,10 +337,12 @@ static char *generator (const char *text, int state)
     if (which == 1) {
 	match = table_key_completions (text, state);
 
-	if (!match) {
+	if (!match && !completed) {
 	    which = 2;
 	    state = 0;
-	}
+	} else {
+            completed = 1;
+        }
     }
 
     /* Try to complete a filename. */
@@ -590,7 +595,14 @@ static void describe (lua_State *L, int index)
     index = absolute (L, index);    
     type = lua_type (L, index);
     
-    if (type == LUA_TNUMBER) {
+    if (luaL_getmetafield (L, index, "__tostring")) {
+	lua_pushvalue (L, index);
+	lua_pcall (L, 1, 1, 0);
+	s = (char *)lua_tolstring (L, -1, &n);
+	lua_pop (L, 1);
+	
+	dump_string (s, n);
+    } else if (type == LUA_TNUMBER) {
 	/* Copy the value to avoid mutating it. */
 	
 	lua_pushvalue (L, index);
@@ -709,15 +721,8 @@ static void describe (lua_State *L, int index)
 		      COLOR(7), COLOR(8), lua_topointer (L, index));
 	dump_string (s, n);
     } else if (type == LUA_TUSERDATA) {
-	if (luaL_getmetafield (L, index, "__tostring")) {
-	    lua_pushvalue (L, index);
-	    lua_pcall (L, 1, 1, 0);
-	    s = (char *)lua_tolstring (L, -1, &n);
-	    lua_pop (L, 1);
-	} else {
-	    n = asprintf (&s, "<%suserdata:%s %p>",
-			  COLOR(7), COLOR(8), lua_topointer (L, index));
-	}
+	n = asprintf (&s, "<%suserdata:%s %p>",
+		      COLOR(7), COLOR(8), lua_topointer (L, index));
 	
 	dump_string (s, n);
     } else if (type == LUA_TTHREAD) {
